<?php

namespace Service;

use Util\Instance\GeneralInstance;
use Util\Instance\ServiceInstance;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\Events\PreDispatch\Listener;
use Exception\Login as LoginException;

class RequestListenerOAuth extends Listener
{
    const OAUTH_HEADER = "OAuth-Token";
    const OAUTH_SECRET_ID = "OAuth-SecretId";
    const OAUTH_KEEP_CONNECTED = "OAuth-KeepConnected";
    const PUBLIC_ROUTES_PATH = "/../Json/publicRoutes.json";
    const KEEP_CONNECTED_POSITIVE = "Yes";

    public function preDispatch(Request $request)
    {
        try {
            $token = GeneralInstance::getKernel()->getHttpRequest()->getHeaders()->get(self::OAUTH_HEADER);
            if (!is_null($token)) {
                $this->checkAccessAndSetSession($token);
            }
            $this->checkPublicRoute($request);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }

    /**
     * @param $token
     */
    private function checkAccessAndSetSession($token)
    {
        $session  = GeneralInstance::getSession();
        $secretId = GeneralInstance::getKernel()->getHttpRequest()->getHeaders()->get(self::OAUTH_SECRET_ID);
        $keepConn = GeneralInstance::getKernel()->getHttpRequest()->getHeaders()->get(self::OAUTH_KEEP_CONNECTED);
        $keepConn = $keepConn == self::KEEP_CONNECTED_POSITIVE ? true : false;
        $access   = ServiceInstance::getOAuth()->checkAccess($token, $secretId, array(), $keepConn);

        if (!empty($access)) {
            if($session->getId() != $access['sessionId']) {
				$session->start();
                $session->destroy();
            }
            $session->setId($access['sessionId']);
            $session->start();
        }
    }

    /**
     * @param Request $request
     * @throws \Exception
     */
    private function checkPublicRoute(Request $request)
    {
        if (!in_array($request->getRoutePath(), $this->readPublicRoutes())) {
            LoginException::accessDenied();
        }
    }

    private function readPublicRoutes()
    {
        $urls = file_get_contents(__DIR__ . self::PUBLIC_ROUTES_PATH);
        return json_decode($urls, true);
    }
}